@@ -13,19 +13,18 @@
 #
 # See the README file for information on usage and redistribution.
 #
+from __future__ import annotations
 
 import io
-import sys
+from types import TracebackType
 
 from . import ContainerIO
 
-##
-# A file object that provides read access to a given member of a TAR
-# file.
 
+class TarIO(ContainerIO.ContainerIO[bytes]):
+    """A file object that provides read access to a given member of a TAR file."""
 
-class TarIO(ContainerIO.ContainerIO):
-    def __init__(self, tarfile, file):
+    def __init__(self, tarfile: str, file: str) -> None:
         """
         Create file object.
 
@@ -35,15 +34,16 @@ class TarIO(ContainerIO.ContainerIO):
         self.fh = open(tarfile, "rb")
 
         while True:
-
             s = self.fh.read(512)
             if len(s) != 512:
-                raise IOError("unexpected end of tar file")
+                msg = "unexpected end of tar file"
+                raise OSError(msg)
 
             name = s[:100].decode("utf-8")
             i = name.find("\0")
             if i == 0:
-                raise IOError("cannot find subfile")
+                msg = "cannot find subfile"
+                raise OSError(msg)
             if i > 0:
                 name = name[:i]
 
@@ -55,19 +55,19 @@ class TarIO(ContainerIO.ContainerIO):
             self.fh.seek((size + 511) & (~511), io.SEEK_CUR)
 
         # Open region
-        ContainerIO.ContainerIO.__init__(self, self.fh, self.fh.tell(), size)
+        super().__init__(self.fh, self.fh.tell(), size)
 
     # Context manager support
-    def __enter__(self):
+    def __enter__(self) -> TarIO:
         return self
 
-    def __exit__(self, *args):
+    def __exit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
+    ) -> None:
         self.close()
 
-    if sys.version_info.major >= 3:
-
-        def __del__(self):
-            self.close()
-
-    def close(self):
+    def close(self) -> None:
         self.fh.close()
